package RanieriApp;

import java.util.*;
import java.io.*;

import Utilities.Commons;

// si richiede che le eventuali decisioni di MF siano parzialmente ordinate 
// rispetto alla data dinizio, poi al tempo dinizio, poi alla data di decisione, 
// poi al tempo di decisione. 

/**
 * @author Michele Ranieri
 * @modifiche Graziella Cipolletti
 */
public class Decisione extends Commons{
	
	public final static String SEPARATORE="||";
	
	private GregorianCalendar initDate;
	private String idF1;
	private String nameF1;
	private String textF1;
	private int nwF1;
	private String idF2;
	private String nameF2;
	private String textF2;
	private int nwF2;
	private String similarity;
	private String probSimilarity;
	private String inclusion;
	private String probInclusion;
	private String decision;
	private GregorianCalendar decDate;

	//usata per identificare univocamente la coppia di frasi
	public String getKey(){
		String key;
		if(idF1.compareTo(idF2)<0){
			key=idF1+idF2;
		}else{
			key=idF2+idF1;
		}
		return key;
	}
	
	public String getIdF1() {
		return idF1;
	}

	public String getNameF1() {
		return nameF1;
	}

	public String getTextF1() {
		return textF1;
	}

	public int getNwF1() {
		return nwF1;
	}

	public String getIdF2() {
		return idF2;
	}

	public String getNameF2() {
		return nameF2;
	}

	public String getTextF2() {
		return textF2;
	}

	public int getNwF2() {
		return nwF2;
	}

	public String getSimilarity() {
		return similarity;
	}

	public String getProbSimilarity() {
		return probSimilarity;
	}

	public String getInclusion() {
		return inclusion;
	}

	public String getProbInclusion() {
		return probInclusion;
	}

	public String getDecision(){
		return decision;
	}
	
	public GregorianCalendar getInitDate(){
		return initDate;
	}
	
	public GregorianCalendar getDecDate(){
		return decDate;
	}
	
	
	public void setSimilarity(String sim){
		this.similarity=sim;
	}
	
	public void setProbSimilarity(String ps){
		this.probSimilarity=ps;
	}
	
	public void setInclusion(String inc){
		this.inclusion=inc;
	}
	
	public void setProbInclusion(String pi){
		this.probInclusion=pi;
	}
	
	
	public void setF1(String id,String name,String text,int nw){
		this.idF1=id;
		this.nameF1=name;
		this.textF1=text;
		this.nwF1=nw;
	}
	
	public void setF2(String id,String name,String text,int nw){
		this.idF2=id;
		this.nameF2=name;
		this.textF2=text;
		this.nwF2=nw;
	}
	
	public void setInitDate(GregorianCalendar gc){
		this.initDate=gc;
	}
	

	public void setDecDate(GregorianCalendar gc){
		this.decDate=gc;
	}
	
	public void setDecision(String d){
		this.decision=d;
	}
	
	public String toString(){

		String res=stampaData(initDate);
		
		res+=(Decisione.SEPARATORE + idF1);
		res+=(Decisione.SEPARATORE + nameF1);
		res+=(Decisione.SEPARATORE + textF1);
		res+=(Decisione.SEPARATORE + nwF1);

		res+=(Decisione.SEPARATORE + idF2);
		res+=(Decisione.SEPARATORE + nameF2);
		res+=(Decisione.SEPARATORE + textF2);
		res+=(Decisione.SEPARATORE + nwF2);
		
		res+=(Decisione.SEPARATORE + similarity);
		res+=(Decisione.SEPARATORE + probSimilarity);
		res+=(Decisione.SEPARATORE + inclusion);
		res+=(Decisione.SEPARATORE + probInclusion);
		res+=(Decisione.SEPARATORE + decision + Decisione.SEPARATORE);
		
		res+=(stampaData(decDate));
		
		return res;
		
	}
	
	//Restituisce una stringa con la data c in formato dd/mm/yyyy||hh.mm.ss
	public String stampaData(GregorianCalendar c){
		int year, month, date, hour, minute,second;

		year=c.get(Calendar.YEAR);
		month=c.get(Calendar.MONTH)+1;
		date=c.get(Calendar.DATE);
		hour=c.get(Calendar.HOUR_OF_DAY);
		minute=c.get(Calendar.MINUTE);
		second=c.get(Calendar.SECOND);
		
		String res= ((date<10?"0":"")+ date+"/"+ (month<10?"0":"")+month+"/"+year+Decisione.SEPARATORE+
				(hour<10?"0":"")+hour+":"+ (minute<10?"0":"")+minute+":"+ (second<10?"0":"")+second);
		
		//System.out.printf("Anno=%d Mese=%d Giorno=%d Ora=%d Minuto=%d Secondo=%d Res=%s%n",year,month,date,hour,minute,second,res);
		
		return res;
	}

/**
 * Aggiunge a partizioni le decisioni contenute in file
 * 
 * partizioni  un insieme di vettori, ognuno contenente le decisioni che si 
 * riferiscono alla stessa coppia di frasi
 * 
 * */	
	public static boolean parseFile(File file, Hashtable<String, Vector<Decisione>> partizioni){
		
		if(!checkRandomSort(file))
		{
			return false;
		}
		
		
		String riga=null;
		StringTokenizer tok;
		BufferedReader br=null;
		String matricola=file.getName().substring(file.getName().indexOf("_")+1, file.getName().indexOf("."));
		
		try {
			System.out.println("Parsing file: "+file.getCanonicalPath());
		
			//apre il file in lettura
			br=new BufferedReader(new InputStreamReader(new FileInputStream(file)));
				
			riga = br.readLine();

		} catch (IOException e) {
			System.err.println("Errore di accesso al file di input "+ file);
			e.printStackTrace();
			System.err.println();
		}

		int year, month, date, hour, minute,second;
		while(riga != null){
			
			if(riga.trim().endsWith("!"))
			{
				try {
					riga+=br.readLine();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			else if(!riga.contains("{"))
			{
				try {
					riga=br.readLine();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			else
			{
			
				if(riga.contains("! "))
					{
					riga=riga.replaceAll("! ", "");
					}
				
				if(riga.contains("﻿"))
				{
					riga=riga.replaceAll("﻿", "");
				}
				
			try{
				tok=new StringTokenizer(riga, Decisione.SEPARATORE);
				if(tok.hasMoreTokens()){
					Decisione d=new Decisione();
			
					//Init date
					String data=tok.nextToken(); //es: 28/09/2008
					date=Integer.parseInt(data.substring(0, 2));
					month=Integer.parseInt(data.substring(3, 5));
					year=Integer.parseInt(data.substring(6));
				
					data=tok.nextToken(); //es: 20.51.52
					hour=Integer.parseInt(data.substring(0, 2));
					minute=Integer.parseInt(data.substring(3, 5));
					second=Integer.parseInt(data.substring(6));
				
					/**
					 Cipolletti
					 */
					d.setInitDate(new GregorianCalendar(year, month-1, date, hour, minute,second));
					
					String idF1, nameF1, textF1,idF2, nameF2,textF2;
					int nwF1,nwF2;
				
					idF1=tok.nextToken();
					nameF1=tok.nextToken();
					textF1=tok.nextToken();
					nwF1=Integer.parseInt(tok.nextToken());
					
					String [] parole=textF1.trim().split(" ");
					nwF1=parole.length;
					
					idF2=tok.nextToken();
					nameF2=tok.nextToken();
					textF2=tok.nextToken();
					nwF2=Integer.parseInt(tok.nextToken());
				
					parole=textF2.trim().split(" ");
					nwF2=parole.length;
					
					d.setF1(idF1, nameF1, textF1, nwF1);
					d.setF2(idF2, nameF2, textF2, nwF2);

					d.setSimilarity(tok.nextToken());
					d.setProbSimilarity(tok.nextToken());
					d.setInclusion(tok.nextToken());
					d.setProbInclusion(tok.nextToken());
				
					d.setDecision(tok.nextToken());
				
					//Decision date
					data=tok.nextToken(); //es: 28/09/2008
					date=Integer.parseInt(data.substring(0, 2));
					month=Integer.parseInt(data.substring(3, 5));
					year=Integer.parseInt(data.substring(6));
				
					data=tok.nextToken(); //es: 20.51.52
					hour=Integer.parseInt(data.substring(0, 2));
					minute=Integer.parseInt(data.substring(3, 5));
					second=Integer.parseInt(data.substring(6));
				
					/**
					 * Cipolletti
					 */
					d.setDecDate(new GregorianCalendar(year, month-1, date, hour, minute,second));
					
					String key=d.getKey();
					
					//estensione 1
					/*
					 * Eliminare dai file di input tutte le decisioni prese pi di una volta 
					 * dello stesso soggetto sulla stessa coppia di requisiti 
					 * (tale coppia non apparir pi). 
					 * OPZIONALE: eliminare la coppia solo se la decisione  discordante.
					 */
					
					Vector<String> coppie=Commons.matrDecisions.get(matricola);
					
					if(coppie==null)
					{
					//	System.out.println("Non esiste il vettore delle matricole, lo aggiungo");
						//il vettore non esiste quindi creane uno nuovo
						coppie=new Vector();
						coppie.addElement(key);
						Commons.matrDecisions.put(matricola,coppie);
						
						Vector<Decisione> decisioni=partizioni.get(key);
						//se ancora non esisteva la classe di equivalenza la creo
						if (decisioni == null) {
							decisioni=new Vector<Decisione>();
							partizioni.put(key, decisioni);
						}
						
						//aggiungo la decisione alle giusta classe di equivalenza
						// (identificata dagli id delle 2 frasi)
						decisioni.addElement(d);	
					}
					else
					{
						//System.out.println("il vettore delle matricole esiste");
						
						if(!coppie.contains(key))
						{
						//	System.out.println("non contiene la chiave");
							
							//se non contiene la coppia aggiungila
							coppie.addElement(key);
							
							Vector<Decisione> decisioni=partizioni.get(key);
							//se ancora non esisteva la classe di equivalenza la creo
							if (decisioni == null) {
								decisioni=new Vector<Decisione>();
								partizioni.put(key, decisioni);
							}
							
							//aggiungo la decisione alle giusta classe di equivalenza
							// (identificata dagli id delle 2 frasi)
							decisioni.addElement(d);	
						}
						else
						{
							// OPZIONALE: eliminare la coppia solo se la decisione  discordante.
							Vector<Decisione> dec=partizioni.get(key);
							
							for(int h=0;h<dec.size();h++)
							{
								Decisione temp=dec.elementAt(h);
								
								if(temp.getKey().compareTo(key)==0 && d.getDecision().compareTo(temp.getDecision())==0)
								{
									// se la decisione  la stessa mantengo la decisione nel vettore
									Vector<Decisione> decisioni=partizioni.get(key);
									//se ancora non esisteva la classe di equivalenza la creo
									if (decisioni == null) {
										decisioni=new Vector<Decisione>();
										partizioni.put(key, decisioni);
									}
									
									//aggiungo la decisione alle giusta classe di equivalenza
									// (identificata dagli id delle 2 frasi)
									decisioni.addElement(d);
									break;
								}
								
							}
							
							
							
						}
					}
					
					
				}
				
			}catch(Exception e){
				System.err.println("Errore nel formato della decisione: "+riga);
				e.printStackTrace();
				System.err.println();
			}
				
				
			try { //legge la riga successiva
				riga = br.readLine();
			} catch (IOException e) {
				System.err.println("Errore di accesso al file di input "+file);
				e.printStackTrace();
				System.err.println();
			} 
			
			}//else not end with !
		}

		return true;
	}
	
	
	

	private static boolean checkRandomSort(File file) {
		/*
		 * Eliminare i file di input dei soggetti che hanno cliccato su random sort; i.e. determinare se le prime
		 * 10 decisioni prese da un soggetto sono in ordine decrescente in base al contenuto del campo SIMILARITY
		 */
		
		double tempSim=0;
		boolean check=false;
		
		try {
			FileInputStream fis=new FileInputStream(file);
			InputStreamReader isr=new InputStreamReader(fis);
			BufferedReader br=new BufferedReader(isr);
			
			String riga=br.readLine();
			int count=1;
			while(riga!=null && count<=10)
			{
				
				if(riga.trim().endsWith("!"))
				{
						riga+=br.readLine();
				}
				else if(!riga.contains("{"))
				{
						riga=br.readLine();
				}
				else
				{
					if(riga.contains("! "))
						{
						riga=riga.replaceAll("! ", "");
						}
					
					if(riga.contains("﻿"))
					{
						riga=riga.replaceAll("﻿", "");
					}
					
					String[]campi=riga.split("\\|\\|");
					String simS=campi[10].substring(campi[10].indexOf("=")+1).replaceAll(",", ".");
					double sim=Double.parseDouble(simS);
				
					if(sim>=tempSim)
					{
						check=true;
					}
					
					riga=br.readLine();
					count++;
					tempSim=sim;
                 				}
							}
			
			
			br.close();
			isr.close();
			fis.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		return check;
}

	//definisce il criterio di ordinamento delle decisioni (usato per il file MF) 
	public long getTimePosition(){
		return this.initDate.getTimeInMillis()*100000000+this.decDate.getTimeInMillis()/1000;
	}
	
	//restituisce la media delle date contenute nel vettore v 
	public static GregorianCalendar dataMedia(Vector<GregorianCalendar> v){
			long sommaMillis=0;
			long numDate=0;
			
			Enumeration<GregorianCalendar>e=v.elements();
			
			while(e.hasMoreElements()){
				numDate++;
				GregorianCalendar gc=e.nextElement();
				sommaMillis+=gc.getTimeInMillis();
			}
			
			GregorianCalendar dataMedia=new GregorianCalendar();
			dataMedia.setTimeInMillis(sommaMillis/numDate);
			return dataMedia;
	}

	//Aggiunge la decisione d al vettore v 
	// in ordine di data (e ora) di inizio e poi di decisione
	public static void aggiungiOrdinato(Vector<Decisione> v, Decisione d) {
		
		if(v.size()==0){
			v.add(d);
		}else{
		
			long timePos=d.getTimePosition();
		
			int pos;
			for(pos=0;pos<v.size() && timePos<v.get(pos).getTimePosition();pos++);
		
			v.insertElementAt(d,pos);
			
		}
		
	}
}
